# -*- coding: utf-8 -*-
import os
import xbmcaddon
from xbmcvfs import translatePath

import sqlite3
from sqlite3 import OperationalError
import json 

current_version = 10

def open_db():
    global db, version
    addon = xbmcaddon.Addon()
    addon_userdata_dir = translatePath(addon.getAddonInfo('profile')) 
    if not os.path.isdir(addon_userdata_dir):
        os.mkdir(addon_userdata_dir)
    db = sqlite3.connect(addon_userdata_dir + 'details.db', timeout = 20)
    db.execute('CREATE TABLE IF NOT EXISTS version (version INTEGER PRIMARY KEY)')
    db.execute('CREATE TABLE IF NOT EXISTS details (id VARCHAR(255) PRIMARY KEY, cover VARCHAR(255), title VARCHAR(255), description VARCHAR(255), cast VARCHAR(255), directors VARCHAR(255), year VARCHAR(255), episodeNumber INTEGER, episodeName VARCHAR(255), seasonNumber INTEGER, episodesInSeason INTEGER, seasonName VARCHAR(255), seriesName VARCHAR(255), genres VARCHAR(255), contentType VARCHAR(255), czech INTEGER)')
    row = None
    for row in db.execute('SELECT version FROM version'):
        version = row[0]
    if not row:
        db.execute('INSERT INTO version VALUES (?)', [current_version])
        db.commit()     
        version = current_version
    if version != current_version:
        version = migrate_db(version)
    db.commit()     

def close_db():
    global db
    db.close()    

def migrate_db(version):
    global db
    if version == 4:
        version = 5
        db.execute('ALTER TABLE epg ADD COLUMN startTime INTEGER')
        db.execute('ALTER TABLE epg ADD COLUMN endTime INTEGER')
        db.execute('ALTER TABLE epg ADD COLUMN channel INTEGER')
        db.execute('ALTER TABLE epg ADD COLUMN title INTEGER')
        db.execute('UPDATE version SET version = ?', str(version))
        db.commit()       
    if version == 5:
        version = 6
        db.execute('DELETE FROM epg_details WHERE description=\'\' AND cover=\'\'')
        db.execute('UPDATE version SET version = ?', str(version))
        db.commit()
    if version == 6:
        version = 7
        try:
          db.execute('ALTER TABLE epg ADD COLUMN availableTo INTEGER')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN episodeNumber INTEGER')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN episodeName VARCHAR(255)')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN seasonNumber INTEGER')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN episodesInSeason INTEGER')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN seasonName VARCHAR(255)')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN seriesName VARCHAR(255)')
        except OperationalError:
          pass
        try:
          db.execute('ALTER TABLE epg_details ADD COLUMN contentType VARCHAR(255)')
        except OperationalError:
          pass
        for row in db.execute('SELECT epgId, ratings FROM epg_details'):
          epgId = row[0]
          ratings = json.loads(row[1])
          for rating in ratings:
            ratings[rating] = ratings[rating] * 10
          db.execute('UPDATE epg_details SET ratings = ? WHERE epgId = ? ', [json.dumps(ratings),str(epgId)])
        db.commit()    
        db.execute('UPDATE epg SET availableTo = startTime + (60*60*24*7)')
        db.commit()    
        db.execute('UPDATE version SET version = ?', str(version))
        db.commit()          
    return version

def get_details(id, api):
    global db
    open_db()

    row = None
    cached = True
    for row in db.execute('SELECT id, cover, title, description, "cast", directors, year, episodeNumber, episodeName, seasonNumber, episodesInSeason, seasonName, seriesName, genres, contentType, czech FROM details WHERE id = ?', [id]):
        id = row[0]
        cover = row[1]
        title = row[2]
        description = row[3]
        cast = json.loads(row[4])
        directors = json.loads(row[5])
        year = row[6]
        episodeNumber = row[7]
        episodeName = row[8]
        seasonNumber = row[9]
        episodesInSeason = row[10]
        seasonName = row[11]
        seriesName = row[12] 
        genres = json.loads(row[13])
        contentType = row[14]
        czech = row[15]
    if not row:
        cached = False
        data = api.express_content(id)
        cover = ''
        title = ''
        description = ''
        cast = []
        directors = []
        year = -1
        episodeNumber = -1
        episodeName = ''
        seasonNumber = -1
        episodesInSeason = -1
        seasonName = ''
        seriesName = ''
        genres = []
        contentType = ''
        czech = 0

        cover = data['images']['tile']
        title = data['titles']['short']
        description = data['summaries']['short']
        if 'credits' in data:
            if 'cast' in data['credits']:
                cast = data['credits']['cast']
            if 'writers' in data['credits']:
                directors = data['credits']['writers']
        year = data['releaseYear']
        episodeNumber = -1
        episodeName = ''
        seasonNumber = -1
        episodesInSeason = -1
        seasonName = ''
        seriesName = ''
        contentType = ''
        for item in data['edits']:
          if 'audioTracks' in item:
              for audiotrack in item['audioTracks']:
                  if audiotrack['language'] == 'cs-CZ':
                      czech = 1
          if 'textTracks' in item and czech == 0:
              for texttrack in item['textTracks']:
                  if texttrack['language'] == 'cs-CZ':
                      czech = -1
          genres = item['genres']
        db.execute('INSERT INTO details VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', (id, cover, title, description, json.dumps(cast), json.dumps(directors), year, episodeNumber, episodeName, seasonNumber, episodesInSeason, seasonName, seriesName, json.dumps(genres), contentType, czech))      
    db.commit()
    close_db()
    details = { 'id' : id, 'title' : title, 'cover' : cover, 'description' : description, 'cast' : cast, 'directors' : directors, 'year' : year, 'episodeNumber' : episodeNumber, 'episodeName' : episodeName, 'seasonNumber' : seasonNumber, 'episodesInSeason' : episodesInSeason, 'seasonName' : seasonName, 'seriesName' : seriesName, 'genres' : genres ,'contentType' : contentType, 'czech' : czech }
    return details, cached
